@@ -16,12 +16,15 @@
 
 #include <set>
 
-#include "../lib/include/catch.hpp"
-#include "../lib/include/picosha2.hpp"
+#include <catch2/catch_test_macros.hpp>
+#include <catch2/catch_session.hpp>
+#include <catch2/matchers/catch_matchers.hpp>
+
 #include "calculate_bucket.hpp"
 #include "disk.hpp"
 #include "plotter_disk.hpp"
 #include "prover_disk.hpp"
+#include "serialize.hpp"
 #include "sort_manager.hpp"
 #include "verifier.hpp"
 
@@ -143,7 +146,6 @@ TEST_CASE("SliceInt64FromBytesFull")
     CHECK(Util::SliceInt64FromBytesFull(bytes, 7, 64) == ((0x1020304050607080ull << 3) | 0b100));
     CHECK(Util::SliceInt64FromBytesFull(bytes, 8, 64) == 0x0203040506070809ull);
 }
-
 TEST_CASE("Util")
 {
     SECTION("Increment and decrement")
@@ -479,6 +481,135 @@ TEST_CASE("F functions")
     }
 }
 
+TEST_CASE("(De)Serialization")
+{
+    Serializer serializer;
+    Deserializer deserializer(serializer.Data());
+    SECTION("Basics")
+    {
+        serializer.Reset();
+        for (uint8_t i = 0; i < 0xFF; ++i) {
+            serializer << i;
+        }
+        uint8_t n{0}, n_deserialized;
+        do{
+            deserializer >> n_deserialized;
+            REQUIRE(n_deserialized == n++);
+        } while (!deserializer.End());
+        serializer.Reset();
+        deserializer.Reset();
+        std::vector<float> vecFloat;
+        REQUIRE_THROWS(deserializer >> vecFloat); // Unexpected vector, leads to read out of bounds
+        float f = 0.123, f_deserialized = 0;
+        double d = 1.23, d_deserialized = 0;
+        int64_t i = -123, i_deserialized = 0;
+        uint64_t u = 123, u_deserialized = 0;
+        serializer << f;
+        deserializer >> f_deserialized;
+        REQUIRE(f_deserialized == f);
+        serializer << d;
+        deserializer >> d_deserialized;
+        REQUIRE(d_deserialized == d);
+        serializer << i;
+        deserializer >> i_deserialized;
+        REQUIRE(i_deserialized == i);
+        serializer << u;
+        deserializer >> u_deserialized;
+        REQUIRE(u_deserialized == u);
+        REQUIRE(deserializer.End());
+        deserializer.Reset();
+        deserializer >> f_deserialized;
+        deserializer >> d_deserialized;
+        deserializer >> i_deserialized;
+        deserializer >> u_deserialized;
+        REQUIRE(f_deserialized == f);
+        REQUIRE(d_deserialized == d);
+        REQUIRE(i_deserialized == i);
+        REQUIRE(u_deserialized == u);
+        REQUIRE(deserializer.End());
+        REQUIRE_THROWS(deserializer >> f_deserialized); // Read out of bounds
+    }
+    SECTION("vector")
+    {
+        std::vector<uint64_t> vec1{1,2,3}, vec2;
+        std::vector<std::vector<uint64_t>> vec3{{1}, {2,3}, {4,5,6}}, vec4;
+        serializer << vec1;
+        serializer << vec2;
+        serializer << vec3;
+        serializer << vec4;
+        std::vector<uint64_t> vec1_deserialized, vec2_deserialized;
+        std::vector<std::vector<uint64_t>> vec3_deserialized, vec4_deserialized;
+        deserializer >> vec1_deserialized;
+        deserializer >> vec2_deserialized;
+        deserializer >> vec3_deserialized;
+        deserializer >> vec4_deserialized;
+        REQUIRE(vec1_deserialized == vec1);
+        REQUIRE(vec2_deserialized == vec2);
+        REQUIRE(vec3_deserialized == vec3);
+        REQUIRE(vec4_deserialized == vec4);
+        REQUIRE(deserializer.End());
+    }
+    SECTION("string")
+    {
+        std::string str1 = "123", str2;
+        serializer << str1;
+        serializer << str2;
+        std::string str1_deserialized, str2_deserialized;
+        deserializer >> str1_deserialized;
+        deserializer >> str2_deserialized;
+        REQUIRE(str1_deserialized == str1);
+        REQUIRE(str2_deserialized == str2);
+        REQUIRE(deserializer.End());
+    }
+    SECTION("DiskProver")
+    {
+        std::string filename = "prover_test.plot";
+        DiskPlotter plotter = DiskPlotter();
+        uint8_t memo[5] = {1, 2, 3, 4, 5};
+        plotter.CreatePlotDisk(
+            ".", ".", ".", filename, 18, memo, 5, plot_id_1, 32, 11, 0, 4000, 2);
+        DiskProver prover1(filename);
+        std::vector<uint8_t> vecBytes = prover1.ToBytes();
+        DiskProver prover2(vecBytes);
+        REQUIRE(prover1.GetFilename() == prover2.GetFilename());
+        REQUIRE(prover1.GetSize() == prover2.GetSize());
+        REQUIRE(prover1.GetId() == prover2.GetId());
+        REQUIRE(prover1.GetMemo() == prover2.GetMemo());
+        vector<unsigned char> hash_input = intToBytes(0, 4);
+        vector<unsigned char> hash(picosha2::k_digest_size);
+        picosha2::hash256(hash_input.begin(), hash_input.end(), hash.begin(), hash.end());
+        vector<LargeBits> qualities1 = prover1.GetQualitiesForChallenge(hash.data());
+        LargeBits proof1 = prover1.GetFullProof(hash.data(), 0);
+        vector<LargeBits> qualities2 = prover2.GetQualitiesForChallenge(hash.data());
+        LargeBits proof2 = prover2.GetFullProof(hash.data(), 0);
+        REQUIRE(qualities1 == qualities2);
+        REQUIRE(proof1 == proof2);
+        vecBytes[0] = 0x02; // Change version
+        REQUIRE_THROWS(DiskProver(vecBytes)); // Invalid version
+        REQUIRE(remove(filename.c_str()) == 0);
+    }
+}
+
+TEST_CASE("K Sizes")
+{
+    Verifier verifier = Verifier();
+    uint8_t challenge[32];
+
+    SECTION("Minimum K Size")
+    {
+        LargeBits result = verifier.ValidateProof(plot_id_1, kMinPlotSize - 1, challenge, nullptr, 0);
+        REQUIRE(result.GetSize() == 0);
+    }
+
+    SECTION("Maximum K Size")
+    {
+        uint8_t *proof_data = new uint8_t[200 * 8];
+        LargeBits result = verifier.ValidateProof(plot_id_1, 200, challenge, proof_data, 200 * 8);
+        REQUIRE(result.GetSize() == 0);
+        delete[] proof_data;
+    }
+}
+
 void HexToBytes(const string& hex, uint8_t* result)
 {
     for (unsigned int i = 0; i < hex.length(); i += 2) {
@@ -509,6 +640,7 @@ void TestProofOfSpace(
         picosha2::hash256(hash_input.begin(), hash_input.end(), hash.begin(), hash.end());
         vector<LargeBits> qualities = prover.GetQualitiesForChallenge(hash.data());
         Verifier verifier = Verifier();
+
         for (uint32_t index = 0; index < qualities.size(); index++) {
             LargeBits proof = prover.GetFullProof(hash.data(), index);
             proof.ToBytes(proof_data);
@@ -965,6 +1097,38 @@ TEST_CASE("BufferedDisk")
     remove("test_file.bin");
 }
 
+TEST_CASE("DiskProver")
+{
+    SECTION("Move constructor")
+    {
+        std::string filename = "prover_test_with_a_long_name_to_avoid_sso.plot";
+        DiskPlotter plotter = DiskPlotter();
+        std::vector<uint8_t> memo{1, 2, 3};
+        plotter.CreatePlotDisk(
+            ".", ".", ".", filename, 18, memo.data(),
+            memo.size(), plot_id_1, 32, 11, 0,
+            4000, 2);
+        DiskProver prover1(filename);
+        auto* p1_filename_ptr = prover1.GetFilename().data();
+        auto* p1_memo_ptr = prover1.GetMemo().data();
+        auto* p1_id_ptr = prover1.GetId().data();
+        auto* p1_table_begin_pointers_ptr = prover1.GetTableBeginPointers().data();
+        auto* p1_C2_ptr = prover1.GetC2().data();
+        DiskProver prover2(std::move(prover1));
+        REQUIRE(prover2.GetFilename().data() == p1_filename_ptr);
+        REQUIRE(prover2.GetMemo().data() == p1_memo_ptr);
+        REQUIRE(prover2.GetId().data() == p1_id_ptr);
+        REQUIRE(prover2.GetTableBeginPointers().data() == p1_table_begin_pointers_ptr);
+        REQUIRE(prover2.GetC2().data() == p1_C2_ptr);
+        REQUIRE(prover1.GetFilename().empty());
+        REQUIRE(prover1.GetMemo().empty());
+        REQUIRE(prover1.GetId().empty());
+        REQUIRE(prover1.GetSize() == prover1.GetSize());
+        REQUIRE(prover1.GetTableBeginPointers().empty());
+        REQUIRE(prover1.GetC2().empty());
+    }
+}
+
 TEST_CASE("FilteredDisk")
 {
     FileDisk d = FileDisk("test_file.bin");
@@ -1025,3 +1189,9 @@ TEST_CASE("FilteredDisk")
 */
     remove("test_file.bin");
 }
+
+int main(int argc, char* argv[])
+{
+    int result = Catch::Session().run(argc, argv);
+    return result;
+}
